<?php

namespace ModulesGarden\DomainOrdersExtended\App\Libs\Api\ENom\Method;

use \ModulesGarden\DomainOrdersExtended\Core\Helper\DomainHelper;
use \ModulesGarden\DomainOrdersExtended\App\Helper\ApiLoger;

/**
 * Description of CheckMoreDomains
 *
 * @author Rafał Ossowski <rafal.os@modulesgarden.com>
 */
class CheckMoreDomains extends AbstractMethod
{

    public function execute($records)
    {
        $returnData = [];
        $domainList = $this->getDomainList($records);

        if (count($domainList) > 30)
        {
            while (!empty($domainList))
            {
                $returnData = array_merge(
                        $returnData, $this->checkDomainList(array_slice($domainList, 0, 30))
                );
                array_splice($domainList, 0, 30);
            }
        }
        else
        {
            $returnData = $this->checkDomainList($domainList);
        }

        return $returnData;
    }

    protected function getDomainList($records = [])
    {
        $domainList = [];
        foreach ($records as $record)
        {
            $domainList[] = $this->idnaConvert->encode(strtolower($record['sld'])) . "."
                    . $this->idnaConvert->encode(strtolower($record['tld']));
        }

        return $domainList;
    }

    protected function checkDomainList($domainList)
    {
        $requestData = [
            'Command'      => 'check',
            'DomainList'   => str_replace(' ', '', implode(',', $domainList)),
            'ResponseType' => 'xml',
            'UID'          => $this->settings['username'],
            'PW'           => $this->settings['password']
        ];

        $response = $this->request->post($requestData);

        $requestData['UID'] = 'xxx';
        $requestData['PW']  = 'xxx';
        ApiLoger::debugModeLogAPI('Enom', $requestData, $response);

        if ($response && $response->isSuccess())
        {
            $response = $response->getBody();

            if ($response->Done == 'true' && count($response->errors) > 0)
            {
                foreach ($response->errors as $apiErrorMessage)
                {
                    throw new \Exception($apiErrorMessage);
                }
            }
            elseif ($response->Done == 'true')
            {
                return $this->formatResponse($response);
            }
        }
        else
        {
            throw new \Exception('No valid response received');
        }

        return false;
    }

    protected function formatResponse($response)
    {
        $result = [];
        foreach ($response as $key => $value)
        {
            switch ($key)
            {
                case 'DomainCount':
                case 'IsPremiumName':
                case 'PremiumPrice':
                case 'PremiumAboveThresholdPrice':
                case 'Command':
                case 'ErrCount':
                case 'ErrX':
                case 'Done':
                    continue;
                default :
                    $this->parserResponse($result, $this->convertDomain($key, $value, $response));
            }
        }

        return $result;
    }

    protected function convertDomain($key, $value, $response = null)
    {
        $name = $this->getName($key);
        if (strpos($key, 'Domain') !== false)
        {
            $did   = str_replace('Domain', '', $key);
            $key   = 'Domain';
            $value = new DomainHelper($this->idnaConvert->decode($value));
            if (is_numeric($did) && $did > 0 && ($response->{'IsPremiumName' . $did} === true || $response->{'IsPremiumName' . $did} === 'true'))
            {
                $premiumPricing = [
                    'TransferPrice'                 => $response->{'TransferPrice' . $did},
                    'RenewalPrice'                  => $response->{'RenewalPrice' . $did},
                    'ExpectedCustomerSuppliedPrice' => $response->{'ExpectedCustomerSuppliedPrice' . $did},
                    'RegistrationFee'               => $response->{'RegistrationFee' . $did},
                    'PremiumPrice'                  => $response->{'PremiumPrice' . $did},
                    'MinPeriod'                     => $response->MinPeriod,
                    'MaxPeriod'                     => $response->MaxPeriod
                ];
            }
        }
        elseif (strpos($key, 'RRPCode') !== false)
        {
            $key   = 'RRPCode';
            $value = $this->choseResult($value);
        }
        else
        {
            $key = null;
        }

        return [
            'name'           => $name,
            'key'            => $key,
            'value'          => $value,
            'premiumPricing' => $premiumPricing
        ];
    }

    protected function choseResult($value)
    {
        switch ($value)
        {
            case 210:
                return self::RESPONSE_RESULT_FREE;
            case 211:
                return self::RESPONSE_RESULT_TAKEN;
            default :
                return self::RESPONSE_RESULT_ERROR;
        }
    }

    protected function getName($key)
    {
        return substr($key, (strpos($key, 'Domain') !== false) ? 6 : 7);
    }

    protected function parserResponse(&$result, $data)
    {
        switch ($data['key'])
        {
            case 'Domain':
                $result[$data['name']]['sld']            = $data['value']->getDomain();
                $result[$data['name']]['tld']            = $data['value']->getTLDWithDot();
                $result[$data['name']]['premiumPricing'] = $data['premiumPricing'];
                break;
            case 'RRPCode':
                $result[$data['name']]['status']         = $data['value'];
                break;
        }
    }
}
