<?php

namespace ModulesGarden\DomainOrdersExtended\App\Libs\Api\ENom\Method;

use \ModulesGarden\DomainOrdersExtended\App\Helper\ApiLoger;

/**
 * Description of CheckOneDomain
 *
 * @author Rafał Ossowski <rafal.os@modulesgarden.com>
 */
class CheckOneDomain extends AbstractMethod
{
    protected $domain;
    protected $tld;

    public function execute($params)
    {
        $this->domain = str_replace(' ', '', $params['sld']);
        $this->tld    = $params['tld'];
        return $this->checkOneDomain();
    }

    public function checkOneDomain()
    {
        $requestData = [
            'Command'      => 'check',
            'SLD'          => $this->idnaConvert->encode(strtolower($this->domain)),
            'TLD'          => $this->idnaConvert->encode(strtolower(substr($this->tld, 1))),
            'ResponseType' => 'xml',
            'UID'          => $this->settings['username'],
            'PW'           => $this->settings['password'],
        ];        
        
        $response = $this->request->post($requestData);

        $requestData['UID'] = 'xxx';
        $requestData['PW']  = 'xxx';
        ApiLoger::debugModeLogAPI('Enom', $requestData, $response);        
        
        return $this->formatResponse($response);
    }

    protected function formatResponse($response)
    {
        $resultData = [];

        if ($response->Done == 'true' && count($response->errors) > 0)
        {
            foreach ($response->errors as $apiErrorMessage)
            {
                throw new \Exception($apiErrorMessage);
            }
        }
        elseif ($response && $response->isSuccess())
        {
            $response = $response->getBody();
            $status   = ($response->Done == 'true') ? $this->getStatus((int) $response->RRPCode) : self::RESPONSE_RESULT_ERROR;
            if ($response->IsPremiumName === 'true' || $response->IsPremiumName === true)
            {
                $premiumPricing = [
                    'TransferPrice'                 => $response->TransferPrice,
                    'RenewalPrice'                  => $response->RenewalPrice,
                    'ExpectedCustomerSuppliedPrice' => $response->ExpectedCustomerSuppliedPrice,
                    'RegistrationFee'               => $response->RegistrationFee,
                    'PremiumPrice'                  => $response->PremiumPrice,
                    'MinPeriod'                     => $response->MinPeriod,
                    'MaxPeriod'                     => $response->MaxPeriod
                ];
            }
        }
        else
        {
            $status = self::RESPONSE_RESULT_ERROR;
        }

        $resultData[] = [
            'sld'            => $this->domain,
            'tld'            => $this->tld,
            'status'         => $status,
            'premiumPricing' => $premiumPricing
        ];

        return $resultData;
    }

    protected function getStatus($rRPCode)
    {
        $status = self::RESPONSE_RESULT_ERROR;
        switch ($rRPCode)
        {
            case 211:
                $status = self::RESPONSE_RESULT_TAKEN;
                break;
            case 210:
                $status = self::RESPONSE_RESULT_FREE;
                break;
        }
        return $status;
    }
}
