<?php

namespace ModulesGarden\DomainOrdersExtended\App\Libs\Api\ENom\Method;

use \ModulesGarden\DomainOrdersExtended\App\Helper\ApiLoger;
use \ModulesGarden\DomainOrdersExtended\Core\Logger\Entity;
use function \ModulesGarden\DomainOrdersExtended\Core\Helper\errorLog;

/**
 * Description of SuggestionDomain
 *
 * @author Rafał Ossowski <rafal.os@modulesgarden.com>
 */
class SuggestionDomain extends AbstractMethod
{
    protected $domain;
    protected $tlds;
    protected $resultData = [];

    public function execute($params)
    {
        $this->domain = str_replace(' ', '', $params['sld']);
        $this->tlds   = $params['tlds'];

        return $this->checkOneDomain();
    }

    public function checkOneDomain()
    {
        $requestData = [
            'Command'      => 'NameSpinner',
            'sld'          => $this->idnaConvert->encode(strtolower($this->domain)),
            'ResponseType' => 'xml',
            'UID'          => $this->settings['username'],
            'PW'           => $this->settings['password'],
            'MaxResults'   => $this->settings['domainSpinnerGroupLimit'],
        ];

        $response           = $this->request->post($requestData);
        $requestData['UID'] = 'xxx';
        $requestData['PW']  = 'xxx';

        if (!$this->isResponseValid($response, $requestData))
        {
            return [];
        }

        ApiLoger::debugModeLogAPI('Enom', $requestData, $response->getBody());

        return $this->formatResponse($response);
    }

    protected function isResponseValid($response, $requestData)
    {
        if (!$response || !$response->isSuccess())
        {
            errorLog(
                    $apiErrorMessage, ['request' => $requestData], ['submodule' => 'Enom', 'rawResponse' => $response->getBody()], [], [], Entity::LEVEL_HIGHT, null
            );
            
            return false;
        }
        
        if (count($response->errors) > 0)
        {
            foreach ($response->errors as $apiErrorMessage)
            {
                errorLog(
                        $apiErrorMessage, ['request' => $requestData], ['submodule' => 'Enom', 'rawResponse' => $response->getBody()], [], [], Entity::LEVEL_HIGHT, null
                );

                return false;
            }
        }

        return true;
    }

    protected function formatResponse($response)
    {
        if ($response && $response->isSuccess())
        {
            $response = $response->getBody();
            if ($response->Done == 'true')
            {
                $this->buildResponseData($response);
            }
        }

        return $this->resultData;
    }

    protected function buildResponseData($response)
    {
        $originalSld = $this->idnaConvert->encode(strtolower($this->domain));
        foreach ($this->getResponseDomains($response) as $value)
        {
            foreach ($this->tlds as $tld)
            {
                $suggestedSld = strtolower($value->name);
                if ($originalSld === $suggestedSld) 
                {
                    continue;
                }
                
                if (isset($value->{$tld}))
                {
                    $this->resultData[] = [
                        'sld'    => $suggestedSld,
                        'tld'    => "." . $tld,
                        'status' => $value->{$tld} == 'n' ? self::RESPONSE_RESULT_TAKEN : self::RESPONSE_RESULT_FREE
                    ];
                }
            }
        }

        return $this;
    }

    public function getResponseDomains($response)
    {
        $data = [];
        foreach ($response->namespin->domains->domain as $value)
        {
            $data[] = $value->{'@attributes'};
        }
        
        return $data;
    }
}
