<?php

namespace ModulesGarden\DomainOrdersExtended\App\Libs\Api\OpenSRS;

use ModulesGarden\DomainOrdersExtended\Core\Helper;

/**
 * Description of Base
 *
 * @author inbs
 */
class Base
{
    protected $protocol = 'XCP';
    private $_socket = false;
    private $_socketErrorNum = false;
    private $_socketErrorMsg = false;
    private $_socketTimeout = 120;                // seconds
    private $_socketReadTimeout = 120;            // seconds
    protected $_opsHandler;
    public $defaultTlds = array('.com', '.net', '.org');
    public $requiredFields = array();

    public function __construct()
    {
        $this->_verifySystemProperties();
        $this->_opsHandler = new Ops();
    }

    public function __destruct()
    {
        if (is_resource($this->_socket)) {
            fclose($this->_socket);
        }
    }

    private function _verifySystemProperties()
    {
        if (!function_exists('version_compare') || version_compare('4.3', phpversion(), '>=')) {
            $error_message = 'PHP version must be v4.3+ (current version is '.phpversion().') to use "SSL" encryption';
            throw new Exception($error_message);
        } elseif (!function_exists('openssl_open')) {
            $error_message = 'PHP must be compiled using --with-openssl to use "SSL" encryption';
            throw new Exception($error_message);
        }
    }

    public function send_cmd($request)
    {
        // make or get the socket filehandle
        if (!$this->init_socket()) {
            throw new Exception('oSRS Error - Unable to establish socket: ('.$this->_socketErrorNum.') '.$this->_socketErrorMsg);
        }
        $this->send_data($request);
        $data = $this->read_data();
        $num_matches = preg_match('/<item key="response_code">401<\/item>/', $data, $matches);
        if ($num_matches > 0) {
            throw new Exception('oSRS Error - Reseller username or OSRS_KEY is incorrect, please check your config file.');
        }
        return $data;
    }

    private function init_socket()
    {
        if ($this->is_connected()) {
            return true;
        }
        $register = Helper\sl('register');
        $this->_socket = fsockopen($register->registry('CRYPT_TYPE') . '://' . $register->registry('OSRS_HOST'), $register->registry('OSRS_SSL_PORT'), $this->_socketErrorNum, $this->_socketErrorMsg, $this->_socketTimeout);
        if (!$this->_socket) {
            return false;
        } else {
            return true;
        }
    }

    public function is_connected()
    {
        return (is_resource($this->_socket)) ? true : false;
    }

    private function close_socket()
    {
        if (is_resource($this->_socket)) {
            fclose($this->_socket);
        }
    }

    private function read_data()
    {
        $buf = $this->readData($this->_socket, $this->_socketReadTimeout);
        if (!$buf) {
            throw new Exception('oSRS Error - Read buffer is empty.  Please make sure IP is whitelisted in RWI. Check the OSRS_KEY and OSRS_USERNAME in the config file as well.');
            $data = '';
        } else {
            $data = $buf;
        }
        if (!empty($this->osrs_debug)) {
            print_r('<pre>'.htmlentities($data).'</pre>');
        }
        return $data;
    }

    private function send_data($message)
    {
        if (!empty($this->osrs_debug)) {
            print_r('<pre>'.htmlentities($message).'</pre>');
        }
        return $this->writeData($this->_socket, $message);
    }

    private function writeData(&$fh, $msg)
    {
        $register = Helper\sl('register');
        $header = '';
        $len = strlen($msg);
        $signature = md5(md5($msg.$register->registry('OSRS_KEY')).$register->registry('OSRS_KEY'));
        $header .= 'POST / HTTP/1.0'.CRLF;
        $header .= 'Content-Type: text/xml'.CRLF;
        $header .= 'X-Username: '.$register->registry('OSRS_USERNAME').CRLF;
        $header .= 'X-Signature: '.$signature.CRLF;
        $header .= 'Content-Length: '.$len.CRLF.CRLF;
        fputs($fh, $header);
        fputs($fh, $msg, $len);
    }

    private function readHeader($fh, $timeout = 5)
    {
        $header = array();
        $http_log = '';
        $line = fgets($fh, 4000);
        $http_log .= $line;
        if (!preg_match('/^HTTP\/1.1 ([0-9]{0,3}) (.*)\r\n$/', $line, $matches)) {
            throw new Exception('oSRS Error - UNEXPECTED READ: Unable to parse HTTP response code. Please make sure IP is whitelisted in RWI.');
            return false;
        }
        $header['http_response_code'] = $matches[1];
        $header['http_response_text'] = $matches[2];
        while ($line != CRLF) {
            $line = fgets($fh, 4000);
            $http_log .= $line;
            if (feof($fh)) { 
                throw new Exception('oSRS Error - UNEXPECTED READ: Error reading HTTP header.');
                return false;
            }
            $matches = explode(': ', $line, 2);
            if (sizeof($matches) == 2) {
                $header[trim(strtolower($matches[0]))] = $matches[1];
            }
        }
        $header['full_header'] = $http_log;
        return $header;
    }

    private function readData(&$fh, $timeout = 5)
    {
        $len = 0;
        socket_set_timeout($fh, $timeout);
        $header = $this->readHeader($fh, $timeout);
        if (!$header || !isset($header{'content-length'}) || (empty($header{'content-length'}))) {
            throw new Exception('oSRS Error - UNEXPECTED ERROR: No Content-Length header provided! Please make sure IP is whitelisted in RWI.');
        }
        $len = (int) $header{'content-length'};
        $line = '';
        while (strlen($line) < $len) {
            $line .= fread($fh, $len);
            if ($this->_opsHandler->socketStatus($fh)) {
                return false;
            }
        }
        if ($line) {
            $buf = $line;
        } else {
            $buf = false;
        }
        $this->close_socket();
        return $buf;
    }
    public function convertArray2Formatted($type = '', $data = '')
    {
        $resultString = '';
        if ($type == 'json') {
            $resultString = json_encode($data);
        }
        if ($type == 'yaml') {
            $resultString = Spyc::YAMLDump($data);
        }
        return $resultString;
    }

    public function getConfiguredTlds()
    {
        $selected = array();
        $suppliedDefaults = array();
        if (isset($this->dataObject->data->selected) && $this->dataObject->data->selected != '') {
            $selected = explode(';', $this->dataObject->data->selected);
        }
        if (isset($this->dataObject->data->defaulttld) && $this->dataObject->data->defaulttld != '') {
            $suppliedDefaults = explode(';', $this->dataObject->data->defaulttld);
        }
        if (count($selected) > 0) {
            return $selected;
        }
        if (count($suppliedDefaults) > 0) {
            return $suppliedDefaults;
        }
        return $this->defaultTlds;
    }
    public function setDataObject($format, $dataObject)
    {
        $this->dataObject = $dataObject;
        $this->dataFormat = $format;
    }

    public function hasDomain()
    {
        return isset($this->dataObject->data->domain);
    }

    public function getDomain()
    {
        return $this->dataObject->data->domain;
    }

    public function send($dataObject, $returnFullResponse = true)
    {
        if (!is_object($dataObject)) {
            $dataObject = new \stdClass();
        }
        $dataObject->protocol = $this->protocol;
        $dataObject->action = $this->action;
        $dataObject->object = $this->object;
        if (
            isset($dataObject->attributes->domain) &&
            substr_count($dataObject->attributes->domain, '.') > 1
        ) {
            $dataObject->attributes->domain = str_replace('www.', '', $dataObject->attributes->domain);
        }
        $xmlCMD = $this->_opsHandler->encode(json_decode(json_encode($dataObject), true));
        $XMLresult = $this->send_cmd($xmlCMD);
        $arrayResult = $this->_opsHandler->decode($XMLresult);
        if (
            !$arrayResult['is_success'] &&
            $arrayResult['response_code'] != 200 &&
            $arrayResult['response_code'] != 415
        ) {
            $errorInfo = null;
            if (array_key_exists('attributes', $arrayResult)) {
                $errorInfo = $arrayResult['attributes'];
            }
            throw new APIException(
                "oSRS Error Code #{$arrayResult['response_code']}: {$arrayResult['response_text']}.",
                $errorInfo
            );
        }
        if (method_exists($this, 'customResponseHandling')) {
            $arrayResult = $this->customResponseHandling($arrayResult, $returnFullResponse);
        }
        $this->resultFullRaw = $arrayResult;
        if (!$returnFullResponse && isset($arrayResult['attributes'])) {
            $this->resultRaw = $arrayResult['attributes'];
        } else {
            $this->resultRaw = $arrayResult;
        }
        $this->resultFullFormatted = $this->convertArray2Formatted($this->_formatHolder, $this->resultFullRaw);
        $this->resultFormatted = $this->convertArray2Formatted($this->_formatHolder, $this->resultRaw);
    }

    public function _validateObject($dataObject, $requiredFields = null)
    {
        if (is_null($requiredFields)) {
            if (isset($this->requiredFields) && !empty($this->requiredFields)) {
                $requiredFields = $this->requiredFields;
            } else {
                $requiredFields = array();
            }
        }
        if (is_array($requiredFields)) {
            foreach ($requiredFields as $i => $field) {
                if (is_array($field)) {
                    if (!isset($dataObject->$i)) {
                        Exception::notDefined($i);
                    }
                    $this->_validateObject($dataObject->$i, $field);
                } else {
                    if (!isset($dataObject->$field) || !$this->isValidField($dataObject->$field)) {
                        Exception::notDefined($field);
                    }
                }
            }
        }
    }
   
    public function isValidField($value)
    {
        $isArray = is_array($value);
        switch (true) {
            case $isArray && count($value):
            case !$isArray && strlen($value):
                $isValid = true;
                break;
            default:
                $isValid = false;
                break;
        }
        return $isValid;
    }
}
