<?php

namespace ModulesGarden\DomainOrdersExtended\App\Libs\Api\OpenSRS\Method;

use \ModulesGarden\DomainOrdersExtended\App\Helper\ApiLoger;

/**
 * Description of CheckMoreDomains
 *
 * @author Rafał Ossowski <rafal.os@modulesgarden.com>
 */
class CheckMoreDomains extends AbstractMethod
{
    protected $domainList        = [];
    protected $returnData        = [];
    protected $premiumReturnData = [];

    public function execute($records)
    {
        foreach ($this->getDomainList($records) as $domain => $tlds)
        {
            $premiumResult = [];

            $requestData = [
                'func' => 'fastDomainLookup',
                'data' => [
                    'domain'     => $this->idnaConvert->encode(strtolower($domain)),
                    'selected'   => implode(';', $tlds),
                    'alldomains' => implode(';', $tlds)
                ]
            ];

            $response = $this->request->process(
                            'json', json_encode($requestData)
                    )->resultFormatted;

            ApiLoger::debugModeLogAPI('OpenSRS', $requestData, $response);

            try
            {
                $premiumRequestData = [
                    'func' => 'premiumdomain',
                    'data' => [
                        'domain'     => $this->idnaConvert->encode(strtolower($domain)),
                        'selected'   => implode(';', $tlds),
                        'alldomains' => implode(';', $tlds)
                    ]
                ];
                $premiumResponse = $this->request->process(
                                        'json', json_encode($premiumRequestData)
                                )->resultFormatted;
                
                $premiumResult[$this->idnaConvert->encode(strtolower($domain))] = json_decode($premiumResponse)->premium;
                
                ApiLoger::debugModeLogAPI('OpenSRS', $premiumRequestData, $premiumResponse);
            }
            catch (\Exception $exc)
            {
                $premiumResult[$this->idnaConvert->encode(strtolower($domain))] = [];
            }

            $this->premiumReturnData = array_merge($this->premiumReturnData, $premiumResult);
            $this->returnData        = array_merge($this->returnData, json_decode($response));
        }
        return $this->formatResponse();
    }

    protected function getDomainList($records = [])
    {
        $this->domainList = [];
        foreach ($records as $record)
        {
            $this->domainList[$record['sld']][] = "." . $this->idnaConvert->encode(strtolower($record['tld']));
        }

        return $this->domainList;
    }

    protected function formatResponse()
    {
        $returnData = [];
        foreach ($this->returnData as $record)
        {
            $premiumPrice  = null;
            $premiumStatus = $this->choseResult($record->result);
            foreach ($this->premiumReturnData[$this->idnaConvert->decode($record->domain)] as $premium)
            {
                if ($premium->domain == $this->idnaConvert->decode($record->domain) . $this->idnaConvert->decode($record->tld))
                {
                    $premiumPrice  = $premium->price;
                    $premiumStatus = $this->choseResult($premium->status);
                }
            }
            $returnData[] = [
                'sld'            => $this->idnaConvert->decode($record->domain),
                'tld'            => $this->idnaConvert->decode($record->tld),
                'status'         => $premiumStatus,
                'premiumPricing' => $premiumPrice
            ];
        }
        return $returnData;
    }

    protected function choseResult($return)
    {
        switch ($return)
        {
            case 'Available':
            case 'available':
                return self::RESPONSE_RESULT_FREE;
            case 'taken':
            case 'Taken':
                return self::RESPONSE_RESULT_TAKEN;
            default:
                return self::RESPONSE_RESULT_ERROR;
        }
    }
}
