<?php

namespace ModulesGarden\DomainOrdersExtended\App\Libs\Api\OpenSRS\Method;

use \ModulesGarden\DomainOrdersExtended\App\Helper\ApiLoger;

/**
 * Description of CheckOneDomain
 *
 * @author Rafał Ossowski <rafal.os@modulesgarden.com>
 */
class CheckOneDomain extends AbstractMethod
{
    protected $domain;
    protected $tld;

    public function execute($params)
    {
        $this->domain = str_replace(' ', '', $params['sld']);
        $this->tld    = $params['tld'];
        return $this->checkOneDomain();
    }

    public function checkOneDomain()
    {
        $tld = "." . $this->idnaConvert->encode(strtolower($this->tld));

        $requestData = [
            'func' => 'fastDomainLookup',
            'data' => [
                'domain'     => $this->idnaConvert->encode(strtolower($this->domain)),
                'selected'   => $tld,
                'alldomains' => $tld,
            ]
        ];

        $response = $this->request->process(
                        'json', json_encode($requestData)
                )->resultFormatted;

        ApiLoger::debugModeLogAPI('OpenSRS', $requestData, $response);

        try
        {
            $premiumRequestData = [
                'func' => 'premiumdomain',
                'data' => [
                    'domain'     => $this->idnaConvert->encode(strtolower($domain)),
                    'selected'   => $tld,
                    'alldomains' => $tld
                ]
            ];
            $premiumResult      = json_decode($this->request->process(
                                    'json', json_encode($premiumRequestData)
                            )->resultFormatted);

            ApiLoger::debugModeLogAPI('OpenSRS', $premiumRequestData, $premiumResult);
        }
        catch (\Exception $exc)
        {
            $premiumResult = [];
        }

        return $this->formatResponse($response, $premiumResult->premium);
    }

    protected function formatResponse($response, $premiumResult)
    {
        $returnData = [];
        foreach (json_decode($response) as $record)
        {
            $premiumPrice  = null;
            $premiumStatus = $this->choseResult($record->result);
            foreach ($premiumResult as $premium)
            {
                if ($premium->domain == $this->idnaConvert->decode($record->domain) . $this->idnaConvert->decode($record->tld))
                {
                    $premiumPrice  = $premium->price;
                    $premiumStatus = $this->choseResult($premium->status);
                }
            }
            $returnData[] = [
                'sld'            => $this->domain,
                'tld'            => $this->tld,
                'status'         => $premiumStatus,
                'premiumPricing' => $premiumPrice
            ];
        }
        return $returnData;
    }

    protected function choseResult($return)
    {
        switch ($return)
        {
            case 'Available':
            case 'available':
                return self::RESPONSE_RESULT_FREE;
            case 'taken':
            case 'Taken':
                return self::RESPONSE_RESULT_TAKEN;
            default:
                return self::RESPONSE_RESULT_ERROR;
        }
    }
}
