<?php

namespace ModulesGarden\DomainOrdersExtended\App\Libs\Api\OpenSRS\Method;

use \ModulesGarden\DomainOrdersExtended\App\Libs\Api\OpenSRS\Request;
use \ModulesGarden\DomainOrdersExtended\Core\Helper\DomainHelper;

use \ModulesGarden\DomainOrdersExtended\App\Helper\ApiLoger;

/**
 * Description of CheckOneDomain
 *
 * @author Rafał Ossowski <rafal.os@modulesgarden.com>
 */
class SuggestionDomain extends AbstractMethod
{
    protected $domain;
    protected $tlds;
    protected $tld;
    protected $resultData = [];
    protected $suggestedable = [
        ".com",
        ".net",
        ".org",
        ".info",
        ".biz",
        ".us",
        ".mobi"
    ];

    public function execute($params)
    {
        $this->domain = str_replace(' ', '', $params['sld']);
        $this->tlds    = $params['tlds'];
        if (in_array($params['tld'], $this->tlds, true) === false && $params['tld'] != "")
        {
            $this->tlds[]  = $params['tld'];
        }

        return $this->checkOneDomain();
    }

    public function checkOneDomain()
    {
        $requestData = [
            "func"       => "suggestDomain",
            "attributes" => [
                "searchstring"     => $this->idnaConvert->encode(strtolower($this->domain)),
                "service_override" => [
                    "suggestion" => [
                        "tlds"    => $this->suggestedable,
                        "maximum" => (int) $this->settings['spinnerCount'] + 3
                    ]
                ],
                "services"         => [
                    "suggestion"
                ]
            ]
        ];

        $response = $this->request->process(
                        "json", json_encode($requestData)
                )->resultRaw;

        ApiLoger::debugModeLogAPI('OpenSRS', $requestData, $response);
        
        if ($response['is_success'] == 1 && isset($response['attributes']['suggestion']) && $response['attributes']['suggestion']['is_success'] == 1)
        {
            return $this->formatResponse($response);
        }
        else
        {
            return [];
        }
    }
    
    public function convertTlds()
    {
        $tlds = [];
        
        foreach ($this->tlds as $tld)
        {
            $tlds[] = "." . $this->idnaConvert->encode(strtolower($tld));
        }
        
        return $tlds;
    }
    
    protected function formatResponse($response)
    {
        $returnData = [];
        foreach ($response['attributes']['suggestion']['items'] as $record)
        {
            $domain = new DomainHelper($record['domain']);
            
            $returnData[] = [
                'sld'    => $domain->getDomain(),
                'tld'    => $domain->getTLDWithDot(),
                'status' => $this->choseResult($record['status'])
            ];
        }
        return $returnData;
    }


    protected function choseResult($status)
    {
        switch (strtolower($status))
        {
            case 'available':
                return self::RESPONSE_RESULT_FREE;
            case 'taken':
                return self::RESPONSE_RESULT_TAKEN;
            default:
                return self::RESPONSE_RESULT_ERROR;
        }
    }
   
}